<?php
if(!defined('ABSPATH')) die; // Die if accessed directly

class DataQlick__ProductsExport extends DataQlick__Export {

    private $WC_WEIGHT_TYPE;
    private $WC_SIZE_TYPE;
    private $barcode_field_name;

    public function __construct() {
        $this->WC_WEIGHT_TYPE = get_option( 'woocommerce_weight_unit' );
        $this->WC_SIZE_TYPE = get_option( 'woocommerce_dimension_unit' );
        $this->barcode_field_name = get_option('dataqlick_barcode');
    }

    public function display_json_data() {
        $this->init_variables();

        if ( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
            $this->process_post_request();
        }

        if ( get_query_var( 'productid' ) ) {
            $ids = explode(',', get_query_var('productid'));
            $results = $this->get_items_data($ids);
            $result = array(
                'count' => count($results),
                'page' => 1,
                'pagesize' => count($results),
                'pagemax' => 1,
                'results' => $results,
            );
        } else {
            $total_items_count = $this->get_all_items_count();
            $pagemax = ceil( $total_items_count / $this->pagesize );

            $result = array(
                'count' => $total_items_count,
                'page' => $this->page,
                'pagesize' => $this->pagesize,
                'pagemax' => $pagemax,
                'results' => $this->get_items_data(),
            );
        }

        header('Content-Type: application/json');
        echo json_encode( $result, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
    }

    public function process_post_request() {
        try {
            $post_data = json_decode( file_get_contents("php://input"), true );

            if( ! is_array($post_data) ) {
                throw new Exception( 'Bad request' );
            }

            $this->update_product_data( $post_data );
            $result = ['status' => 'ok' ];

        } catch ( Exception $e ) {
            $result = ['error' => $e->getMessage()];
        }

        header('Content-Type: application/json');
        echo json_encode( $result, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE );
        exit;
    }

    public function get_all_items_count() {
        return count(
            wc_get_products( array( 'limit' => -1, 'return' => 'ids' ) )
        );
    }

    public function get_items_data( $product_ids = null ) {

        $wp_products = array();
        $product_type = '';

        if ( ! $product_ids ) {
            $wp_products = wc_get_products(
                array(
                    'limit' => $this->pagesize,
                    'offset' => ($this->page - 1) * $this->pagesize,
                )
            );
        } else {
            foreach ( $product_ids as $product_id ) {
                $wp_products[] = wc_get_product( $product_id );
            }
        }

        $result_products = array();

        foreach ( $wp_products as $key => $product ) {

            $product_type = $product->get_type();

            if ( $product_type == 'variable' ) {

                $variation_ids = $product->get_children();

                foreach ($variation_ids as $variation_id ) {
                    $result_products[] = $this->get_export_data_from_product( wc_get_product( $variation_id ) );
                }
            }
            else {
                $result_products[] = $this->get_export_data_from_product( $product );
            }
        }

        return $result_products;
    }

    public function get_export_data_from_product( $product ) {
        $data = $product->get_data();


        if ( $product->get_type() == 'variation' ) {
            $parent_product = wc_get_product( $product->parent_id );
            $parent_data = $parent_product->get_data();

            if ( $data['manage_stock'] ) {
                $qty = $data['stock_quantity'];
            } else {
                $qty = $parent_data['stock_quantity'];
            }

            $bar_code = (int)$this->get_product_barcode( $parent_product );
            $filtered_category_ids = $this->remove_parent_category_ids_from_all_category_ids( $parent_data['category_ids'] );
            $tags = $this->get_taxonomy_names_list( $parent_data['tag_ids'] );
        }
        else {
            $qty = $data['stock_quantity'];
            $bar_code = (int)$this->get_product_barcode( $product );

            $filtered_category_ids = $this->remove_parent_category_ids_from_all_category_ids( $data['category_ids'] );
            $tags = $this->get_taxonomy_names_list( $data['tag_ids'] );
        }

        $category = $this->get_taxonomy_names_list( $filtered_category_ids );


        return array(
            'id' => (string)$data['id'],
            'name' => $data['name'],
            'description' => $data['description'],
            'sku' => $data['sku'],
            'unitOfMeasure' => 'Each',
            'length' => $data['length'] ? (int)$data['length'] : 0,
            'height' => $data['height'] ? (int)$data['height'] : 0,
            'width' => $data['width'] ? (int)$data['width'] : 0,
            'weight' => $data['weight'] ? (int)$data['weight'] : 0,
            'unitPrice' => (float)$data['price'],
            'qty' => $qty,
            'weightType' => $this->WC_WEIGHT_TYPE,
            'sizeType' => $this->WC_SIZE_TYPE,
            'updated' => $data['date_modified']->__toString(),
            'category' => $category,
            'tags' => $tags,
            'imageUrl' => get_the_post_thumbnail_url( $data['id'], 'full' ),
            'purchaseCost' => (float)$data['regular_price'],
            'barCode' => $bar_code,
            'isActive' => ( $data['status'] === 'publish' ? true : false ),
            'isInventory' => ( $data['virtual'] || $data['downloadable'] || !$data['stock_quantity'] ? false : true ),
            'isTaxable' => $product->is_taxable(),
        );
    }

    private function get_product_barcode( $product ) {
        $barcode = '';

        foreach ( $product->get_data()['attributes'] as $attr ) {
            $attr_data = $attr->get_data();

            if ( $attr_data['name'] === $this->barcode_field_name ) {
                $barcode = $attr_data['value'];
                break;
            }
        }

        return $barcode;
    }

    private function get_taxonomy_names_list( $taxonomy_array ) {
        $string_list = '';

        if ( count($taxonomy_array) > 0 ) {
            foreach ( $taxonomy_array as $id ) {
                if ( strlen($string_list) > 0 ) {
                    $string_list .= ',';
                }
                $string_list .= get_term( $id )->name;
            }
        }

        return $string_list;
    }

    private function remove_parent_category_ids_from_all_category_ids( $category_ids ) {
        $result_cat_ids = $category_ids;

        foreach ( $category_ids as $cat_id ) {
            $cat_data = get_category( $cat_id );

            if ( in_array( $cat_data->parent, $result_cat_ids ) ) {
                $result_cat_ids = array_diff( $result_cat_ids, array($cat_data->parent) );
            }
        }

        return $result_cat_ids;
    }

    private function update_product_data( $new_data ) {
        foreach ($new_data as $data) {

            if ( (int)$data['id'] >= 0) {
                $product = wc_get_product( (int)$data['id'] );
            }

            if ( ! $product ) {
                continue;
            }

            if ( (int)$data['qty'] >= 0) {
                $product->set_stock_quantity( (int)$data['qty'] );
            }

            if ( (int)$data['price'] >= 0 ) {

                if ( $product->get_sale_price() ) {
                    $product->set_sale_price( (string)$data['price'] );
                } else {
                    $product->set_regular_price( (string)$data['price'] );
                }
            }

            if ( (int)$data['qty'] >= 0 || (int)$data['price'] >= 0 ) {
                $product->save();
            }
        }
    }

}


